﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Infragistics.Controls.Menus;
using Infragistics.Windows;

namespace xamMenuFun
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {        
        #region Fields

        private bool _buttonClicked = false;
        private XamMenuItem _lastClicked = null;
        private XamMenuItem _currentHighlighted = null;        
        private List<HierarchicalElementLocation> _locations = new List<HierarchicalElementLocation>();
        private GroupPanelsBusinessLogic _businessLogic;

        #endregion Fields

        #region Constructors

        public MainWindow()
        {
            _businessLogic = new GroupPanelsBusinessLogic();
            this.DataContext = _businessLogic;
            InitializeComponent();
            _businessLogic.GroupPanelList = _businessLogic.GetGroupPanels();
        }

        #endregion Constructors

        #region Methods

        private void WireExpanderEvents(Grid grid, bool wire)
        {
            foreach (UIElement child in grid.Children)
            {
                Expander expander = child as Expander;
                if (expander != null)
                {
                    if (wire)
                    {
                        // Wire event to capture when expanders are expanded/collapsed
                        expander.SizeChanged += new SizeChangedEventHandler(expander_SizeChanged);
                    }
                    else
                    {
                        // Unwire event to capture when expanders are expanded/collapsed
                        expander.SizeChanged -= new SizeChangedEventHandler(expander_SizeChanged);
                    }

                    // ASSUMPTION: There can be two and only two levels of collapsable panels. 
                    //             Therefore only need to check this expander for child expanders
                    //             and no need for recursion.                    
                    object[] childExpanders = UIElementTreeHelper.FindControls(expander, typeof(Expander), 0);
                    if (childExpanders != null && childExpanders.Length > 0)
                    {
                        // Get the Grid the child expanders are part of
                        Grid childGrid = expander.Content as Grid;
                        if (childGrid != null)
                        {
                            foreach (Expander childExpander in childExpanders)
                            {
                                if (wire)
                                {
                                    // Wire event to capture when expanders are expanded/collapsed
                                    childExpander.SizeChanged += new SizeChangedEventHandler(expander_SizeChanged);
                                }
                                else
                                {
                                    // Unwire event to capture when expanders are expanded/collapsed
                                    childExpander.SizeChanged -= new SizeChangedEventHandler(expander_SizeChanged);
                                }
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Get the location of all expanders.
        /// </summary>
        /// <param name="grid"></param>
        private void ProcessLocationData(Grid grid)
        {
            double location = 0;
            List<HierarchicalElementLocation> childLocations = null;

            foreach (UIElement child in grid.Children)
            {
                Expander expander = child as Expander;
                if (expander != null)
                {
                    // ASSUMPTION: There can be two and only two levels of collapsable panels. 
                    //             Therefore only need to check this expander for child expanders
                    //             and no need for recursion.                    
                    object[] childExpanders = UIElementTreeHelper.FindControls(expander, typeof(Expander), 0);
                    
                    if (childExpanders != null && childExpanders.Length > 0)
                    {
                        // Get the Grid the child expanders are part of
                        Grid childGrid = expander.Content as Grid;
                        if (childGrid != null)
                        {
                            childLocations = new List<HierarchicalElementLocation>();
                            foreach (Expander childExpander in childExpanders)
                            {
                                RowDefinition childRowDef = childGrid.RowDefinitions[Grid.GetRow(childExpander)];
                                // The child element uses the same location as its parent. The offset will be used to position.
                                HierarchicalElementLocation childLocation = new HierarchicalElementLocation(childExpander.Name, location, childRowDef.Offset);
                                childLocations.Add(childLocation);
                            }
                        }
                    }

                    RowDefinition rowDef = grid.RowDefinitions[Grid.GetRow(expander)];
                    _locations.Add(childLocations == null ? new HierarchicalElementLocation(expander.Name, location, rowDef.Offset) : new HierarchicalElementLocation(expander.Name, location, rowDef.Offset, childLocations));
                    location += rowDef.ActualHeight;

                    // Child locations might have been set by a prior Expander, reset to null
                    childLocations = null;
                }
            }
        }

        private void ResetAllPanelLocations()
        {
            Grid svGrid = _scrollViewer.Content as Grid;
            if (svGrid != null)
            {
                _locations.Clear();
                ProcessLocationData(svGrid);
            }
        }

        private int GetLocationIndex()
        {
            int index = 0;
            for (int i = 0; i < _locations.Count; i++)
            {
                if (_scrollViewer.VerticalOffset >= _locations[i].Location)
                    index = i;
                else
                    break;
            }

            return index;
        }

        #endregion Methods

        #region Event handlers

        void expander_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            // Need to reset all panel locations
            ResetAllPanelLocations();
        }

        /// <summary>
        /// Not currently wired to anything
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void xamNavigator_ItemClicked(object sender, Infragistics.Controls.Menus.ItemClickedEventArgs e)
        {
            //XamMenuItem mi = e.Item;
            //mi.IsSubmenuOpen = false;
            GroupPanelData item = e.Item.Header as GroupPanelData;
            
            MessageBox.Show("Item clicked: " + item.Title);
        }

        private void NavigatorItemLabel_Loaded(object sender, RoutedEventArgs e)
        {
            XamMenuItem xmi = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(XamMenuItem), true) as XamMenuItem;

            // Set tooltip
            TextBlock textBlock = sender as TextBlock;
            ToolTipService.SetInitialShowDelay(textBlock, 1000);            
            textBlock.ToolTip = textBlock.Text;

            // Wire events -- click means navigate to group panel, SubmenuOpened is to keep drop down from happening
            xmi.Click += new EventHandler(xmi_Click);
            xmi.SubmenuOpened +=new EventHandler(xmi_SubmenuOpened);
            xmi.SubmenuClosed += new EventHandler(xmi_SubmenuClosed);
            

        }

        void xmi_SubmenuClosed(object sender, EventArgs e)
        {
            _buttonClicked = false;
        }

        private void DecorateMenu(XamMenuItem mi)
        {
            // Decorate the correct menut item, if it is a child,
            // decorate its parent.
            _currentHighlighted = GetMenuItemToHighlight(mi);
            _currentHighlighted.FontWeight = System.Windows.FontWeights.Bold;

            // Place the selected parent indicator 
            SetSelectedIndicator(_currentHighlighted, System.Windows.Visibility.Visible);
       
        }

        private void SetSelectedIndicator(XamMenuItem mi, Visibility visibility)
        {
            IList<FrameworkElement> elements = UIElementTreeHelper.GetDescendantsFromType(_currentHighlighted, typeof(TextBlock));
            foreach (FrameworkElement element in elements)
            {
                if (element.Name.Equals("_currentSelection"))
                {
                    element.Visibility = visibility;
                    break;
                }
            }
        }

        private void UndecorateMenuItem(XamMenuItem mi)
        {
            mi.FontWeight = System.Windows.FontWeights.Normal;

            // Remove the selected parent indicator
            SetSelectedIndicator(mi, System.Windows.Visibility.Hidden);
        }

         /// <summary>
        /// If the panel at the locationIndex is enabled and collapsed, expand it.
        /// </summary>
        /// <param name="uiElementIndex"></param>        
        private void ExpandGroupPanel(int uiElementIndex)
        {
            ExpandGroupPanel(uiElementIndex, -1);
        }

        /// <summary>
        /// If the panel at the locationIndex is enabled and collapsed, expand it. 
        /// Expand the child panel as indicated by the passed index.
        /// </summary>
        /// <param name="uiElementIndex"></param>
        /// <param name="childIndex">If not -1, process child</param>
        private void ExpandGroupPanel(int uiElementIndex, int childIndex)
        {            
            Grid svGrid = _scrollViewer.Content as Grid;
            if (svGrid != null)
            {
                UIElementCollection elements = svGrid.Children;
                UIElement uiElement = elements[uiElementIndex];
                Expander groupPanel = uiElement as Expander;

                // If the group panel is enabled and collapsed, expand it
                if (groupPanel.IsEnabled)
                {
                    if (!groupPanel.IsExpanded)
                    {
                        groupPanel.IsExpanded = true;
                        ResetAllPanelLocations();
                    }

                    if (childIndex != -1)
                    {
                        // ASSUMPTION: There can be two and only two levels of collapsable panels. 
                        //             Therefore only need to check this expander for child expanders
                        //             and no need for recursion.  

                        // Process child index
                        object[] childExpanders = UIElementTreeHelper.FindControls(groupPanel, typeof(Expander), 0);

                        if (childExpanders != null && childExpanders.Length > 0)
                        {
                            Expander childGroupPanel = childExpanders[childIndex] as Expander;
                            if (childGroupPanel.IsEnabled && !childGroupPanel.IsExpanded)
                            {
                                childGroupPanel.IsExpanded = true;
                                ResetAllPanelLocations();
                            }
                        }
                    }

                }
            }
        }       

        void xmi_Click(object sender, EventArgs e)
        {
            XamMenuItem mi = sender as XamMenuItem;

            // Only if we've clicked something new
            if (_lastClicked == null || (_lastClicked != null && _lastClicked != mi))
            {
                // Reset the last clicked item to normal appearance
                if (_currentHighlighted != null)
                {
                    // Reset prior selection
                    UndecorateMenuItem(_currentHighlighted);
                }

                // Need to decorate the clicked menu item to indicate it is active                               
                DecorateMenu(mi);

                // Expand if collapsed


                // Reset last clicked
                _lastClicked = mi;
                
                // Scroll to correct spot
                GroupPanelData item = mi.Header as GroupPanelData;
                //MessageBox.Show("Clicked: " + item.Title + " location: [" + item.ParentLocationIndex.ToString() + "," + item.LocationIndex.ToString() + "]");

                HierarchicalElementLocation parentLocation = null;
                HierarchicalElementLocation childOffset = null;               

                // Scroll
                if (item.HasParent)
                {
                    // If the parent is collapsed, expand it (so we can see the child) 
                    ExpandGroupPanel(item.ParentLocationIndex, item.LocationIndex); 
                
                    parentLocation = _locations[item.ParentLocationIndex];
                    childOffset = parentLocation.Children[item.LocationIndex];
                    _scrollViewer.ScrollToVerticalOffset(parentLocation.Location + childOffset.Offset);
                }
                else
                {
                    // If the panel is collapsed, expand it
                    ExpandGroupPanel(item.LocationIndex);

                    // Don't be confused, this is a parent but use its LocationIndex.
                    parentLocation = _locations[item.LocationIndex];
                    _scrollViewer.ScrollToVerticalOffset(parentLocation.Location);
                }
            }
        }

        void xmi_SubmenuOpened(object sender, EventArgs e)
        {            
            XamMenuItem mi = sender as XamMenuItem;
            if (mi.HasChildren && !_buttonClicked)
            {                
                mi.IsSubmenuOpen = false;
                xmi_Click(sender, e);
            }

            // Reset the button flag if it was true this go around
            if (_buttonClicked)
            {
                _buttonClicked = false;
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            // Need a way to keep the xmi_SubmenuOpened event from processing
            // This is klugey way for now.
            _buttonClicked = true;

            // Drop down submenu
            XamMenuItem xmi = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(XamMenuItem), true) as XamMenuItem;
            
            if (xmi.HasChildren)
            {
                xmi.IsSubmenuOpen = true;
            }
        }

        private void NavigatorSubitem_Loaded(object sender, RoutedEventArgs e)
        {
            XamMenuItem xmi = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(XamMenuItem), true) as XamMenuItem;

            // Set tooltip
            TextBlock textBlock = sender as TextBlock;
            ToolTipService.SetInitialShowDelay(textBlock, 1000);
            textBlock.ToolTip = textBlock.Text;

            // Wire events -- click means navigate to group panel, SubmenuOpened is to keep drop down from happening
            xmi.Click += new EventHandler(xmi_Click);
        }

        private void _scrollViewer_Loaded(object sender, RoutedEventArgs e)
        {
            if (this.IsLoaded)
            {
                ScrollViewer scrollViewer = sender as ScrollViewer;
                Grid svGrid = scrollViewer.Content as Grid;

                // Going to want to use the RowDefinition for the location
                // as the Expander is padded at top. Need a complex object
                // since expanders can be nested.
                if (svGrid != null)
                {
                    // Process all Expander locations, after method finishes
                    // execution, the _location variable will have all
                    // the Expander locations.
                    ProcessLocationData(svGrid);

                    // Wire expander events
                    WireExpanderEvents(svGrid, true);
                }
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {            
            // Make first menu item selected
            IList<FrameworkElement> menuItems = UIElementTreeHelper.GetDescendantsFromType(_navigatorMenu, typeof(XamMenuItem));
            XamMenuItem xmi = menuItems[0] as XamMenuItem;
            DecorateMenu(xmi);
            _lastClicked = xmi;
            _currentHighlighted = xmi;

            // Need to wire here after the scroll viewer has been loaded
            _scrollViewer.SizeChanged += new SizeChangedEventHandler(_scrollViewer_SizeChanged);
            _scrollViewer.ScrollChanged += new ScrollChangedEventHandler(_scrollViewer_ScrollChanged); 
 
            
            
        }

        private void _scrollViewer_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            if (_emptyPanel != null)
                _emptyPanel.Height = _scrollViewer.ViewportHeight;
        }

        private void _scrollViewer_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            // Determine what should be highlighted based on where the scroll is
            int locationIndex = GetLocationIndex();

            // Get the top level menu items
            XamMenu menu = _navigatorMenu;                        
            IList<FrameworkElement> menuItems = UIElementTreeHelper.GetDescendantsFromType(menu, typeof(XamMenuItem));

            // Get the menu item that should be highlighted
            XamMenuItem mi = menuItems[locationIndex] as XamMenuItem;            

            if (_currentHighlighted != null && _currentHighlighted != mi)
            {
                // Reset prior selection
                UndecorateMenuItem(_currentHighlighted);
            }

            DecorateMenu(mi);            
            _currentHighlighted = mi;

            // If the last clicked was a sub-menu item under the menu item 
            // referenced by mi, don't reset the LastClicked
            GroupPanelData childData = _lastClicked.Header as GroupPanelData;
            GroupPanelData parentData = mi.Header as GroupPanelData;
            if (childData.ParentLocationIndex != parentData.LocationIndex)
            {
                _lastClicked = mi;
            }
        }

        /// <summary>
        /// Returns the last highlighted menu item. If a sub-menu item was
        /// the last clicked, this will return its parent, otherwise
        /// will return the passed menu item.
        /// </summary>
        /// <param name="xmi"></param>
        /// <returns></returns>
        private XamMenuItem GetMenuItemToHighlight(XamMenuItem xmi)
        {
            XamMenuItem mi = xmi;
            GroupPanelData passedData = xmi.Header as GroupPanelData;
            if (passedData.HasParent)
            {
                // Get the parent as that is the one to highlight
                IList<FrameworkElement> menuItems = UIElementTreeHelper.GetDescendantsFromType(_navigatorMenu, typeof(XamMenuItem));
                mi = menuItems[passedData.ParentLocationIndex] as XamMenuItem;
            }
            
            return mi;
        }

        private void XamContextMenu_Opening(object sender, OpeningEventArgs e)
        {
            XamContextMenu contextMenu = sender as XamContextMenu;            
            contextMenu.Items.Clear();

            // Get all menu items
            XamMenu menu = _navigatorMenu;
            IList<FrameworkElement> menuItems = UIElementTreeHelper.GetDescendantsFromType(menu, typeof(XamMenuItem));

            // For each menu item add a context menu item
            XamMenuItem contextMenuItem;
            foreach (FrameworkElement element in menuItems)
            {
                XamMenuItem mi = element as XamMenuItem;
                GroupPanelData data = mi.Header as GroupPanelData;
                contextMenuItem = new XamMenuItem();
                contextMenuItem.Header = data.Title;
                contextMenuItem.IsCheckable = true;
                contextMenuItem.StaysOpenOnClick = true;
                contextMenuItem.IsChecked = true; // TODO: this will be set based on what are currently visible
                contextMenuItem.Tag = data;
                contextMenu.Items.Add(contextMenuItem);                
            }

            // Add a seperator
            XamMenuSeparator separator = new XamMenuSeparator();
            separator.Style = null; // Need this or else exceptions
            contextMenu.Items.Add(separator);            

            // Add the 'Close' menu item
            contextMenuItem = new XamMenuItem();
            contextMenuItem.Header = "Close";            
            contextMenuItem.StaysOpenOnClick = false;
            contextMenuItem.Click += new EventHandler(contextMenuItem_Click);
            contextMenu.Items.Add(contextMenuItem);
        }

        void contextMenuItem_Click(object sender, EventArgs e)
        {
            XamMenuItem item = sender as XamMenuItem;

            IList<GroupPanelData> selectedList = new List<GroupPanelData>();            
            for(int i = 0; i < _navigatorContextMenu.Items.Count; i++)
            {
                XamMenuItem menuItem = _navigatorContextMenu.Items[i] as XamMenuItem;
                if (menuItem != null)
                {
                    if (menuItem.IsChecked)
                    {
                        selectedList.Add(menuItem.Tag as GroupPanelData);
                    }
                }
            }

            StringBuilder sb = new StringBuilder();
            foreach (GroupPanelData data in selectedList)
            {
                sb.AppendLine("\t" + data.Title);
            }
            string msg = string.Format("You have just selected:\n{0}", sb.ToString());
            MessageBox.Show(msg);
        }

        #endregion Event handlers
    }
}
